/*
 * This contains roughly the same structure as the main decoder,
 * except that instead of executing the instruction, we write 
 * an assembler-like representation of it to a passed buffer.
 *
 */

#include "sim.h"
#include "pretty_tables.h"

typedef unsigned int (*opprettydecode_f)(unsigned int, char *);

opprettydecode_f prettydecodetable[] = {
  NULL,
  prettyop_pmdmr,
  prettyop_dmwim,
  prettyop_dmrwim,
  prettyop_dmrw,
  prettyop_pmrw,
  prettyop_loadregim,
  prettyop_loadnondataregim,
  prettyop_alumove,
  prettyop_alu,
  prettyop_jump,
  prettyop_dountil,
  prettyop_shiftdmrw,
  prettyop_shiftpmrw,
  prettyop_shiftmove,
  prettyop_shiftim,
  prettyop_shift,
  prettyop_datamove,
  prettyop_modectl,
  prettyop_jumpindirect,
  prettyop_return,
  prettyop_modify,
  prettyop_trap,
  prettyop_divq,
  prettyop_divs,
  prettyop_satmr,
  prettyop_stackctl,
  prettyop_jumpfi,
  prettyop_modifyfl,
  prettyop_iorw,
  prettyop_nop,
  prettyop_idle,
  prettyop_idlen
};



/*
 * Returns the reg/rgp for the register name (of len namelen).
 *
 */
int findreg(char *name, int namelen, int *reg, int *rgp)
{
  int i,j;
  if (!name || (!reg || !rgp))
    return 0;

  for (i=0;i<16;i++)
    {
      for (j=0;j<4;j++)
	{
	  if (!strncmp(name, regs[i][j], namelen))
	    {
	      *reg = i;
	      *rgp = j;
	      return 1;
	    }
	}
    }
  
  return 0;
}

unsigned short getconstant(unsigned int yy, unsigned int cc, unsigned int bo)
{
  if(bo==1)
    {
      switch(yy)
	{
	case 0x00:
	  switch(cc)
	    {
	    case 0x00: return 0x0001;
	    case 0x01: return 0x0002;
	    case 0x02: return 0x0004;
	    case 0x03: return 0x0008;
	    }
	case 0x01:
	  switch(cc)
	    {
	    case 0x00: return 0x0010;
	    case 0x01: return 0x0020;
	    case 0x02: return 0x0040;
	    case 0x03: return 0x0080;
	    }
	case 0x02:
	  switch(cc)
	    {
	    case 0x00: return 0x0100;
	    case 0x01: return 0x0200;
	    case 0x02: return 0x0400;
	    case 0x03: return 0x0800;
	    }
	case 0x03:
	  switch(cc)
	    {
	    case 0x00: return 0x1000;
	    case 0x01: return 0x2000;
	    case 0x02: return 0x4000;
	    case 0x03: return 0x8000;
	    }
	}
    }
  else if (bo==3)
    {
      switch(yy)
	{
	case 0x00:
	  switch(cc)
	    {
	    case 0x00: return 0xfffe;
	    case 0x01: return 0xfffd;
	    case 0x02: return 0xfffb;
	    case 0x03: return 0xfff7;
	    }
	case 0x01:
	  switch(cc)
	    {
	    case 0x00: return 0xffef;
	    case 0x01: return 0xffdf;
	    case 0x02: return 0xffbf;
	    case 0x03: return 0xff7f;
	    }
	case 0x02:
	  switch(cc)
	    {
	    case 0x00: return 0xfeff;
	    case 0x01: return 0xfdff;
	    case 0x02: return 0xfbff;
	    case 0x03: return 0xf7ff;
	    }
	case 0x03:
	  switch(cc)
	    {
	    case 0x00: return 0xefff;
	    case 0x01: return 0xdfff;
	    case 0x02: return 0xbfff;
	    case 0x03: return 0x7fff;
	    }
	}
    }
  else
    fprintf(stderr, "pretty: unknown bo\n");
  return 0;
}


/*
 * 
 *
 */
int prettydecode(unsigned int opcode, char *buf)
{
  int type = 0;

  if (!buf)
    return -1;
  
  memset(buf, 0x00, MAXDECODE+1);

  type = getoptype(opcode);

  if (type && (type <= 32))
    return (*(prettydecodetable[type]))(opcode,buf);
  
  sprintf(buf, "pretty: unknown instruction");
  return -1;
}
